#include <stdint.h>
#include <kernel.h>
#include <t_syslog.h>
#include <t_stdlib.h>
#include "syssvc/serial.h"
#include "syssvc/syslog.h"
#include "kernel_cfg.h"
#include "Stream.h"
#include <Adafruit_GFX.h>
#include <Adafruit_ST7735.h>
#include "btf.h"
#include "analogin_api.h"

Adafruit_ST7735 tft(P10_14, P10_15, P10_12, P10_13, P8_15, P8_14);
analogin_t joystick_rl;

extern "C" void lcddelay(int ms)
{
  int count = 24000;
  int i;
  for (i = count*ms; i>0; i--);
}

extern "C" void drawPixel(int16_t x, int16_t y, uint32_t color) {
	tft.drawPixel(x, y, color);
}

extern "C" void drawline(int16_t x0, int16_t y0, int16_t x1, int16_t y1,uint32_t color) {
	int16_t slope = abs(y1 - y0) > abs(x1 - x0);
	if (slope) {
		swap(x0, y0);
		swap(x1, y1);
	}

	if (x0 > x1) {
		swap(x0, x1);
		swap(y0, y1);
	}

	int16_t dx, dy;
	dx = x1 - x0;
	dy = abs(y1 - y0);

	int16_t err = dx / 2;
	int16_t ystep;

	if (y0 < y1) {
		ystep = 1;
	} else {
		ystep = -1;
	}

	for (; x0<=x1; x0++) {
		if (slope) {
			tft.drawPixel(y0, x0, color);
		} else {
			tft.drawPixel(x0, y0, color);
		}
		err -= dy;
		if (err < 0) {
			y0 += ystep;
			err += dx;
		}
	}
}

extern "C" void drawrectangle(uint8_t c1, uint8_t c2, uint8_t c3, uint8_t c4, uint8_t c5, uint8_t c6, uint8_t c7, uint8_t c8, uint8_t c9)
{
	drawline(c1,c2,c3,c4,c9);
	//lcddelay(200);
	drawline(c3,c4,c5,c6,c9);
	//lcddelay(200);
	drawline(c7,c8,c5,c6,c9);
	//lcddelay(200);
	drawline(c1,c2,c7,c8,c9);
	lcddelay(20);
}

extern "C" void blinky_main_task(intptr_t exinf) {

	j_work_enable = false;

	analogin_init(&joystick_rl, P1_11); // jyoistick 初期化

	// Initialize 1.8" TFT
	tft.initR(INITR_REDTAB);   // initialize a ST7735 chip, red tab

	tft.fillScreen(ST7735_BLACK);

	sta_cyc(JOYSTICK_CYC);	// jyoistick 周期ハンドラ起動

  	while(1){
		// 背景、柱作成
 		tft.fillRect(0, 0, ST7735_TFTWIDTH, ST7735_TFTHEIGHT, BLACK);
		pillar();
		// ゲーム処理開始
		while(1){
			if (btf(/*&joystic_rl*/) == 1){
				init();
				break;
			}
		}
	}
}

//ボタン入力取得
uint8_t readButton(analogin_t *obj) {
    float a = analogin_read(obj);

    a *= 5.0;

    printf("Button read analog = %f\r\n",a);
    if (a < 0.2)
    	return BUTTON_DOWN;
    if (a < 1.0)
    	return BUTTON_RIGHT;
    if (a < 1.7)
    	return BUTTON_SELECT;
    if (a < 2.6)
    	return BUTTON_UP;
    if (a < 4.6)
    	return BUTTON_LEFT;
    else
    	return BUTTON_NONE;
}

//ジョイスティック周期割込みハンドラ
extern "C" void joystick_cyc(intptr_t exinf)
{
	uint8_t button_data;

	if(j_work_enable == false)
		return;

	//ボタン入力取得
	button_data = readButton(&joystick_rl);
	//ブロック操作
	joystick_work(button_data);
}

